NAME

Crypt::Passphrase::SaltedHash - Validate against Crypt::SaltedHash hashes with Crypt::Passphrase

VERSION

version 0.02

SYNOPSIS

my $authenticator = Crypt::Passphrase->new(
    encoder    => 'Argon2',
    validators => [ 'SaltedHash' ],
);

...

if (!$authenticator->verify_password($password, $hash)) {
    die "Invalid password";
}
elsif ($authenticator->needs_rehash($hash)) {
    my $new_hash = $authenticator->hash_password($password);
    ...
}

DESCRIPTION

This module implements a validator against hashes generated by Crypt::SaltedHash.

SEE ALSO

Crypt::Passphrase

Crypt::SaltedHash

SOURCE

The development version is on github at https://github.com/robrwo/perl-Crypt-Passphrase-SaltedHash and may be cloned from https://github.com/robrwo/perl-Crypt-Passphrase-SaltedHash.git

SUPPORT

Please report any bugs or feature requests on the bugtracker website https://github.com/robrwo/perl-Crypt-Passphrase-SaltedHash/issues

When submitting a bug or request, please include a test-file or a patch to an existing test-file that illustrates the bug or desired feature.

AUTHOR

Robert Rothenberg <perl@rhizomnic.com>

COPYRIGHT AND LICENSE

This software is Copyright (c) 2026 by Robert Rothenberg <perl@rhizomnic.com>.

This is free software, licensed under:

The Artistic License 2.0 (GPL Compatible)